import sys, os, time

class DBGPMiddleware():
    """WSGI Middleware which loads the PyDBGP debugger.
    
    in development.ini:
    
    [DEFAULT]
    idekey         character key for use with dbgp proxy
    host           machine client debugger (e.g. Komodo IDE) or proxy runs on
    port           port the client debugger or proxy listens on
    breakonexcept  only start debugging when an uncaught exception occurs
    """
        
    def __init__(self, app, config, idekey='', host='127.0.0.1', port='9000',
                 breakonexcept='0'):
        self.app = app
        self.config = config

        self.idekey = idekey
        self.host = host
        self.port = int(port)
        self.brk = bool(int(breakonexcept))

    def __call__(self, environ, start_response):
        """ 
        Debug this request.
        """ 
        from dbgp import client
        if self.brk:
            # breaks on uncaught exceptions
            client.brkOnExcept(self.host, self.port, self.idekey)
        else:
            # breaks on the next executed line
            client.brk(self.host, self.port, self.idekey)

            # we might want to do this, but you end up in some random middleware
            # and it's not the best experience.  Instead, break here, go
            # set some breakpoints where you want to debug, the continue

            #c = client.backendCmd(self.idekey)
            #c.stdin_enabled = 0
            #c.connect(self.host, self.port, 'application', [__file__])
            #
            ## despite it's name, this is how to run a function, it does not
            ## start a thread
            #return c.runThread(self.app, (environ, start_response), {})

        # run the app now, if you've stopped here in a debugger, I suggest
        # you go set some breakpoints and continue to those.
        return self.app(environ, start_response)


def make_dbgp_middleware(app, global_conf, idekey='', host='127.0.0.1', port='9000',
                 breakonexcept='0'):
    """
    Wrap the application in a component that will connect to a dbgp server
    for each request
    """
    return DBGPMiddleware(app, global_conf, idekey, host, port, breakonexcept)

